@@ -61,6 +61,8 @@
 #include <gpgme++/key.h>
 #include <gpgme++/verificationresult.h>
 #include <gpgme++/decryptionresult.h>
+#include <gpgme++/gpgmepp_version.h>
+#include <gpgme++/context.h>
 
 #include <gpg-error.h>
 
@@ -77,6 +79,10 @@
 #include <algorithm>
 #include <sstream>
 
+#if GPGMEPP_VERSION > 0x10B01 // > 1.11.1
+# define GPGME_HAS_LEGACY_NOMDC
+#endif
+
 using namespace Kleo::Crypto;
 using namespace Kleo;
 using namespace GpgME;
@@ -193,7 +199,8 @@ static bool relevantInDecryptVerifyContext(const VerificationResult &r)
 {
     // for D/V operations, we ignore verification results which are not errors and contain
     // no signatures (which means that the data was just not signed)
-    return r.error() || r.numSignatures() > 0;
+
+    return (r.error() && r.error().code() != GPG_ERR_DECRYPT_FAILED) || r.numSignatures() > 0;
 }
 
 static QString signatureSummaryToString(int summary)
@@ -245,14 +252,6 @@ static QString formatValidSignatureWithTrustLevel(const UserID &id)
     }
 }
 
-static QString renderFingerprint(const char *fpr)
-{
-    if (!fpr) {
-        return QString();
-    }
-    return QString::fromLatin1(fpr).toUpper().replace(QRegularExpression("(....)"), "\\1 ").trimmed();
-}
-
 static QString renderKeyLink(const QString &fpr, const QString &text)
 {
     return QStringLiteral("<a href=\"key:%1\">%2</a>").arg(fpr, text);
@@ -263,7 +262,14 @@ static QString renderKey(const Key &key)
     if (key.isNull()) {
         return i18n("Unknown certificate");
     }
-    return renderKeyLink(QLatin1String(key.primaryFingerprint()), renderFingerprint(key.primaryFingerprint()));
+
+    if (key.primaryFingerprint() && strlen(key.primaryFingerprint()) > 16 && key.numUserIDs()) {
+        const QString text = QStringLiteral("%1 (%2)").arg(Formatting::prettyNameAndEMail(key).toHtmlEscaped()).arg(
+            Formatting::prettyID(QString::fromLocal8Bit(key.primaryFingerprint()).right(16).toLatin1().constData()));
+        return renderKeyLink(QLatin1String(key.primaryFingerprint()), text);
+    }
+
+    return renderKeyLink(QLatin1String(key.primaryFingerprint()), Formatting::prettyID(key.primaryFingerprint()));
 }
 
 static QString renderKeyEMailOnlyNameAsFallback(const Key &key)
@@ -358,7 +364,7 @@ static void updateKeys(const VerificationResult &result) {
     // provide Key information in a verification result. The Key object is
     // a dummy just holding the KeyID. This hack ensures that all available
     // keys are fetched from the backend and are populated
-    for (const auto sig: result.signatures()) {
+    for (const auto &sig: result.signatures()) {
         // Update key information
         sig.key(true, true);
     }
@@ -467,7 +473,13 @@ static QString formatDecryptionResultOverview(const DecryptionResult &result, co
 
     if (err.isCanceled()) {
         return i18n("<b>Decryption canceled.</b>");
-    } else if (!errorString.isEmpty()) {
+    }
+#ifdef GPGME_HAS_LEGACY_NOMDC
+    else if (result.isLegacyCipherNoMDC()) {
+        return i18n("<b>Decryption failed: %1.</b>", i18n("No integrity protection (MDC)."));
+    }
+#endif
+    else if (!errorString.isEmpty()) {
         return i18n("<b>Decryption failed: %1.</b>", errorString.toHtmlEscaped());
     } else if (err) {
         return i18n("<b>Decryption failed: %1.</b>", QString::fromLocal8Bit(err.asString()).toHtmlEscaped());
@@ -577,10 +589,22 @@ static QString formatDecryptionResultDetails(const DecryptionResult &res, const
         return details + QLatin1String("<i>") + i18np("One unknown recipient.", "%1 unknown recipients.", res.numRecipients()) + QLatin1String("</i>");
     }
 
+#ifdef GPGME_HAS_LEGACY_NOMDC
+    if (res.isLegacyCipherNoMDC()) {
+        details += i18nc("Integrity protection was missing because an old cipher was used.",
+                         "<b>Hint:</b> If this file was encrypted before the year 2003 it is "
+                         "likely that the file is legitimate.  This is because back "
+                         "then integrity protection was not widely used.") + QStringLiteral("<br/><br/>") +
+                   i18nc("The user is offered to force decrypt a non integrity protected message. With the strong advice to re-encrypt it.",
+                         "If you are confident that the file was not manipulated you should re-encrypt it after you have forced the decryption.") +
+                   QStringLiteral("<br/><br/>");
+    }
+#endif
+
     if (!recipients.empty()) {
         details += i18np("Recipient:", "Recipients:", res.numRecipients());
         if (res.numRecipients() == 1) {
-            return details + renderKey(recipients.front());
+            return details + QLatin1Char(' ') + renderKey(recipients.front());
         }
 
         details += QLatin1String("<ul>");
@@ -722,12 +746,13 @@ std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptResul
 
 std::shared_ptr<DecryptVerifyResult> AbstractDecryptVerifyTask::fromDecryptVerifyResult(const DecryptionResult &dr, const VerificationResult &vr, const QByteArray &plaintext, const AuditLog &auditLog)
 {
+    int err = dr.error() ? dr.error().code() : vr.error().code();
     return std::shared_ptr<DecryptVerifyResult>(new DecryptVerifyResult(
             DecryptVerify,
             vr,
             dr,
             plaintext,
-            0,
+            err,
             QString(),
             inputLabel(),
             outputLabel(),
@@ -926,7 +951,7 @@ class DecryptVerifyTask::Private
 {
     DecryptVerifyTask *const q;
 public:
-    explicit Private(DecryptVerifyTask *qq) : q(qq), m_backend(nullptr), m_protocol(UnknownProtocol)  {}
+    explicit Private(DecryptVerifyTask *qq) : q(qq), m_backend(nullptr), m_protocol(UnknownProtocol), m_ignoreMDCError(false)  {}
 
     void slotResult(const DecryptionResult &, const VerificationResult &, const QByteArray &);
 
@@ -944,6 +969,7 @@ public:
     std::shared_ptr<Output> m_output;
     const QGpgME::Protocol *m_backend;
     Protocol m_protocol;
+    bool m_ignoreMDCError;
 };
 
 void DecryptVerifyTask::Private::emitResult(const std::shared_ptr<DecryptVerifyResult> &result)
@@ -1068,11 +1094,31 @@ static void ensureIOOpen(QIODevice *input, QIODevice *output)
     }
 }
 
+void DecryptVerifyTask::setIgnoreMDCError(bool value)
+{
+    d->m_ignoreMDCError = value;
+}
+
 void DecryptVerifyTask::doStart()
 {
     kleo_assert(d->m_backend);
     try {
         QGpgME::DecryptVerifyJob *const job = d->m_backend->decryptVerifyJob();
+
+#ifdef GPGME_HAS_LEGACY_NOMDC
+        if (d->m_ignoreMDCError) {
+            qCDebug(KLEOPATRA_LOG) << "Modifying job to ignore MDC errors.";
+            auto ctx = QGpgME::Job::context(job);
+            if (!ctx) {
+                qCWarning(KLEOPATRA_LOG) << "Failed to get context for job";
+            } else {
+                const auto err = ctx->setFlag("ignore-mdc-error", "1");
+                if (err) {
+                    qCWarning(KLEOPATRA_LOG) << "Failed to set ignore mdc errors" << err.asString();
+                }
+            }
+        }
+#endif
         kleo_assert(job);
         d->registerJob(job);
         ensureIOOpen(d->m_input->ioDevice().get(), d->m_output->ioDevice().get());
